
from django.forms import Field, ValidationError
from django.utils.translation import ugettext_lazy as _

from assets.images.widgets import ImagesFormFieldWidget
from assets.images.forms import build_images_form_class


class ImagesFormField(Field):

    widget = ImagesFormFieldWidget
    form = None

    def __init__(self, image_model):
        self._image_model = image_model
        super().__init__()

    def init_form(self, *args, **kwargs):

        form_class = build_images_form_class(self._image_model)

        self.form = form_class(*args, **kwargs)
        self.widget.form = self.form

    def clean(self, *args, **kwargs):

        if not self.form.is_valid():
            raise ValidationError(_('Form is invalid.'))

        return self.form.cleaned_data

    def commit(self, *args, **kwargs):
        return self.form.commit(*args, **kwargs)




from django.utils.translation import ugettext_lazy as _

from assets.images.utils import refresh_logo


def refresh_logos(modeladmin, request, queryset):
    for instance in queryset:
        refresh_logo(instance)

refresh_logos.short_description = _('Refresh logos')




from os.path import splitext, join

from django import template
from django.conf import settings


register = template.Library()


@register.simple_tag(takes_context=True)
def static_img(context, path):

    is_webp_supported = context.get('IS_WEBP_SUPPORTED')

    folder = 'webp' if is_webp_supported else 'img'

    if is_webp_supported:
        path = splitext(path)[0] + '.webp'

    return join(settings.STATIC_URL, folder, path)





import json

from django import forms
from django.urls import reverse_lazy
from django.contrib.contenttypes.models import ContentType

from assets.images.utils import refresh_logo


def build_images_form_class(image_model_):

    class ImagesForm(ImagesFormBase):

        image_model = image_model_

        images = forms.ModelMultipleChoiceField(
            image_model.objects.none(), required=False)

        class Meta(ImagesFormBase.Meta):
            model = image_model_.get_parent_model_class()

    return ImagesForm


class ImagesFormBase(forms.ModelForm):

    def __init__(
            self,
            data=None,
            files=None,
            instance=None,
            initial=None,
            **kwargs):

        self._images = instance.images.all() if instance else []

        initial = {'images': [i.id for i in self._images or []]}

        super().__init__(
            data=data,
            files=files,
            instance=instance,
            initial=initial,
            **kwargs)

        if self.is_bound:
            self.fields['images'].queryset = self.image_model.objects.all()

    @property
    def serialized_images(self):
        return json.dumps([
            {'id': i.id, 'url': i.get_preview_url()} for i in self._images
        ])

    @property
    def upload_url(self):
        ct = ContentType.objects.get_for_model(self.image_model)
        return reverse_lazy('images:upload', args=[ct.id])

    def commit(self, instance):

        instance.images.all().update(**{
            self.image_model.parent_field: None
        })

        ordering = {
            img_id: i for i, img_id in enumerate(self.data.getlist('images'))
        }

        for img in self.cleaned_data.get('images') or []:
            setattr(img, img.parent_field, instance)
            img.order = ordering[str(img.id)]
            img.save(update_fields=[img.parent_field, 'order'])

        refresh_logo(instance)

        return instance

    class Media:
        css = {
            'all': [
                'file-manager/file-manager.css',
                'dropzone/dropzone.min.css'
            ]
        }
        js = [
            'dropzone/dropzone.min.js',
            'file-manager/file-manager.js',
        ]

    class Meta:
        fields = ['images']


class UploadImageForm(forms.Form):

    url = forms.URLField(required=False)

    file = forms.ImageField(required=False)




from os.path import join

from cbsettings import DjangoDefaults


class StaticFileSettings(object):

    BOWER_INSTALLED_APPS = ()
    STYLESHEETS = ()
    JAVASCRIPT = ()

    IS_LESS_ENABLED = False

    STATICFILES_STORAGE = 'pipeline.storage.PipelineStorage'

    STATICFILES_FINDERS = DjangoDefaults.STATICFILES_FINDERS + [
        'pipeline.finders.PipelineFinder',
        'djangobower.finders.BowerFinder'
    ]

    STATIC_URL = '/static/'

    @property
    def PIPELINE(self):
        return {
            'JS_COMPRESSOR': 'pipeline.compressors.jsmin.JSMinCompressor',
            'CSS_COMPRESSOR': 'pipeline.compressors.cssmin.CSSMinCompressor',
            'COMPILERS': self.COMPILERS,
            'STYLESHEETS': {
                'generic': {
                    'source_filenames': self.STYLESHEETS,
                    'output_filename': 'cache/generic.css',
                }
            },
            'JAVASCRIPT': {
                'generic': {
                    'source_filenames': self.JAVASCRIPT,
                    'output_filename': 'cache/generic.js'
                }
            }
        }

    @property
    def STATIC_ROOT(self):
        return join(self.BASE_DIR, 'static-collect')

    @property
    def BOWER_COMPONENTS_ROOT(self):
        return join(self.BASE_DIR, 'static')

    @property
    def STATICFILES_DIRS(self):
        return [
            join(self.BASE_DIR, 'static'),
            join(self.BASE_DIR, 'static', 'bower_components')
        ]

    @property
    def COMPILERS(self):
        if self.IS_LESS_ENABLED:
            return ['pipeline.compilers.less.LessCompiler']
        return []

    @property
    def INSTALLED_APPS(self):
        return super().INSTALLED_APPS + [
            'django.contrib.staticfiles',
            'pipeline',
            'djangobower'
        ]

    @property
    def MIDDLEWARE(self):
        return super().MIDDLEWARE + [
            'pipeline.middleware.MinifyHTMLMiddleware'
        ]

    @property
    def TEMPLATES(self):
        templates = super().TEMPLATES

        for template in templates:
            template['OPTIONS']['context_processors'] += [
                'django.template.context_processors.static'
            ]

        return templates


class MediaFileSettings(object):

    MEDIA_URL = '/media/'

    FILE_UPLOAD_PERMISSIONS = 0o755

    THUMBNAIL_QUALITY = 85

    IS_WEBP_ENABLED = False
    IS_IMAGES_APP_ENABLED = False

    @property
    def MEDIA_ROOT(self):
        return join(self.BASE_DIR, 'media')

    @property
    def INSTALLED_APPS(self):

        installed_apps =  super().INSTALLED_APPS + [
            'django_cleanup',
            'sorl.thumbnail',
            'assets'
        ]

        if self.IS_IMAGES_APP_ENABLED:
            installed_apps.append('assets.images')

        return installed_apps

    @property
    def TEMPLATES(self):
        templates = super().TEMPLATES

        extra_processors = ['django.template.context_processors.media']

        if self.IS_WEBP_ENABLED:
            extra_processors.append('assets.context_processors.webp')

        for template in templates:
            template['OPTIONS']['context_processors'] += extra_processors

        return templates




from assets.webp import is_webp_supported


def webp(request):

    is_supported = is_webp_supported(request)

    return {
        'IMG_FORMAT': 'WEBP' if is_supported else 'JPEG',
        'IS_WEBP_SUPPORTED': is_supported
    }
